// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.io;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.io.common.CdmDefaultExport;
import eu.etaxonomy.cdm.io.common.IExportConfigurator;
import eu.etaxonomy.cdm.io.common.IIoConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator;
import eu.etaxonomy.cdm.io.sdd.out.SDDExportConfigurator;
import eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>
 * ExportHandler class.
 * </p>
 *
 * @author n.hoffmann
 * @created Sep 11, 2009
 * @version 1.0
 */
public class ExportManager extends AbstractIOManager<IExportConfigurator> {

	/**
	 *
	 * @param applicationConfiguration
	 */
	private ExportManager(ICdmApplicationConfiguration applicationConfiguration) {
		super(applicationConfiguration);
	}

	/**
	 * <p>
	 * NewInstance
	 * </p>
	 *
	 * @param applicationConfiguration
	 *            a
	 *            {@link eu.etaxonomy.cdm.api.application.CdmApplicationController}
	 *            object.
	 * @return a {@link eu.etaxonomy.taxeditor.io.ExportManager} object.
	 */
	public static ExportManager NewInstance(
			ICdmApplicationConfiguration applicationConfiguration) {
		return new ExportManager(applicationConfiguration);
	}

	/**
	 * <p>
	 * createIOJob
	 * </p>
	 *
	 * @param configurator
	 *            a {@link eu.etaxonomy.cdm.io.common.IExportConfigurator}
	 *            object.
	 * @return a {@link org.eclipse.core.runtime.jobs.Job} object.
	 */
	@Override
	protected Job createIOJob(final IExportConfigurator configurator) {

		Assert.isNotNull(configurator, "Configuration may not be null");

		final Display display = Display.getCurrent();

		Job job = new Job("Export: " + configurator.getClass().getSimpleName()) {
			@Override
			protected IStatus run(IProgressMonitor monitor) {
				monitor.beginTask(
						"Exporting database. This will take some time.", 100);
				monitor.worked(10);

			    /** see ticket # 4456 and */
//				display.asyncExec(new Runnable() {
//
//					@Override
//					public void run() {
//
//					    // terminate any open transactions
//						IConversationEnabled activePart = (IConversationEnabled) StoreUtil
//								.getActivePage().getActivePart();
//						activePart.getConversationHolder().commit(false);
//					}
//				});

//				monitor.worked(10);

				CdmDefaultExport<IExportConfigurator> exporter = new CdmDefaultExport<IExportConfigurator>();
				configurator.setProgressMonitor(CdmProgressMonitorAdapter
						.CreateSubMonitor(monitor, 80));
				exporter.setCdmAppController(applicationConfiguration);
				monitor.worked(10);

				try {
					exporter.invoke(configurator);
					monitor.worked(60);
				} catch (RuntimeException e) {
					MessagingUtils.messageDialog("Error exporting data", this,
							"An error occured while"
									+ "exporting to destination '"
									+ configurator.getDestinationNameString()
									+ "'.\n"
									+ "Please check error log for details.", e);
				}

				display.asyncExec(new Runnable() {

					@Override
					public void run() {
						IWorkbenchPart activePart = StoreUtil.getActivePage()
								.getActivePart();
						if (activePart instanceof IConversationEnabled) {
							// terminate any open transactions
							IConversationEnabled conversationEnabled = (IConversationEnabled) activePart;
							// restarting transaction and committing it to
							// trigger
							// change listener
							// TODO verify correct behaviour

							try{
								conversationEnabled.getConversationHolder()
									.startTransaction();

								conversationEnabled.getConversationHolder()
									.commit();
							}catch(RuntimeException e){
								MessagingUtils.messageDialog("Error starting conversation handling", this, "" +
										"Conversation Error: "+ e);
							}
						}
					}
				});
				monitor.worked(10);

				return Status.OK_STATUS;
			}
		};

		return job;
	}

	/**
	 * @param jaxb
	 * @return
	 */
	private IIoConfigurator getConfigurator(TYPE type) {
		Assert.isNotNull(type, "Type should not be null");

		switch (type) {
		case Jaxb:
			return JaxbExportConfigurator.NewInstance(null, null);
		case Sdd:
			return SDDExportConfigurator.NewInstance(null, null, null);
		default:
			MessagingUtils.notImplementedMessage(this);
			throw new IllegalArgumentException("Export not supported yet");
		}
	}

	/**
	 * <p>
	 * JaxbConfigurator
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator} object.
	 */
	public final JaxbExportConfigurator JaxbConfigurator() {
		return (JaxbExportConfigurator) getConfigurator(TYPE.Jaxb);
	}

	/**
	 * <p>
	 * SddConfigurator
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.io.sdd.out.SDDExportConfigurator}
	 *         object.
	 */
	public final SDDExportConfigurator SddConfigurator() {
		return (SDDExportConfigurator) getConfigurator(TYPE.Sdd);
	}

}
