//$Id$

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.lang.reflect.Field;
import java.text.Collator;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.dialogs.FilteredItemsSelectionDialog;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.IEnumTerm;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
* <p>Abstract AbstractFilteredCdmEnumSelectionDialog class.</p>
*
* @author c.mathew
* @created 18.07.2013
* @version 1.0
*/
public abstract class AbstractFilteredCdmEnumSelectionDialog<T extends IEnumTerm> extends
		FilteredItemsSelectionDialog implements IConversationEnabled {

	private ConversationHolder conversation;

	protected List<T> model;	
	private String settings;	
	
	
	
	/**
	 * <p>Constructor for AbstractFilteredCdmResourceSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation 
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 * @param cdmEnum a T object.
	 * @param <T> a T object.
	 */
	protected AbstractFilteredCdmEnumSelectionDialog(Shell shell, 
			ConversationHolder conversation, 
			String title, 
			boolean multi, 
			String settings,
			T cdmEnum) {
		super(shell, multi);
		setTitle(title);
		setMessage("Use * for wildcard, or ? to see all entries");
		this.settings = settings;
		
		this.conversation = conversation;
		
		init();
		
		initModel();
		
		String objectTitle = getTitle(cdmEnum);
		if (objectTitle != null) {
			setInitialPattern(objectTitle);
		}
		
		setListLabelProvider(createListLabelProvider());
		setDetailsLabelProvider(createDetailsLabelProvider());
		
		setSelectionHistory(new ResourceSelectionHistory());
	}
	
	/**
	 * By default, we are returning the standard list label provider
	 * 
	 * Override in subclasses if you want different behavior 
	 * 
	 * @return
	 */
	protected ILabelProvider createDetailsLabelProvider() {
		return createListLabelProvider();
	}

	/**
	 * 
	 * @return
	 */
	protected ILabelProvider createListLabelProvider() {
		return new FilteredCdmResourceLabelProvider();
	}

	/**
	 * Override in subclasses.
	 * Will run before initModel()
	 */
	protected void init() {
		
	}
	
	/**
	 * <p>getSelectionFromDialog</p>
	 *
	 * @param dialog a {@link eu.etaxonomy.taxeditor.ui.dialog.selection.AbstractFilteredCdmResourceSelectionDialog} object.
	 * @param <TYPE> a TYPE object.
	 * @return a TYPE object.
	 */
	protected static <TYPE extends IEnumTerm> TYPE getSelectionFromDialog(AbstractFilteredCdmEnumSelectionDialog<TYPE> dialog) {
		
		int result = dialog.open();
		
		if (result == Window.CANCEL) {
			return null;
		}
		
		return dialog.getSelectedEnumTerm();
	}


	/**
	 * <p>getTitle</p>
	 *
	 * @param cdmObject a T object.
	 * @return a {@link java.lang.String} object.
	 */
	protected String getTitle(T cdmEnum) {
		return cdmEnum.getMessage();
	}
	

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		initModel();		
		super.refresh();
	}
	
	/**
	 * <p>initModel</p>
	 */
	abstract protected void initModel();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createFilter()
	 */
	/** {@inheritDoc} */
	@Override
	protected ItemsFilter createFilter() {
		return new ItemsFilter() {

			/**
			 * Always returns false to enforce refiltering even if the pattern is equal
			 */
			@Override
			public boolean equalsFilter(ItemsFilter filter) {
				return false;
			}
			
			@Override
			public boolean isConsistentItem(Object item) {
				return false;
			}

			@Override
			public boolean matchItem(Object item) {
				String text = null;
				if(item instanceof IEnumTerm){
					text = ((IEnumTerm) item).getMessage();
				}else if(item instanceof String){
					text = (String) item;
				}
				return text != null ? matches(text) : false;
			}
			
		};
	}
	

	/**
	 * Set the filter input to the Agent's title cache
	 *
	 * @param cdmObject a T object.
	 */
	protected void setPattern(T cdmObject) {
		// FilteredSelection does some very tricky caching to make sure it 
		// runs with high performance. 
		// This works for most use cases, but we want to change the model while the dialog is open
		// and all the clever caching prevents the content provider from knowing that the model has changed
		// I am aware, that this is a hack, but the FilteredSelectionDialog API does not offer a convenient 
		// way to solve the problem.
		try {
			Field lastCompletedFilter = this.getClass().getSuperclass().getSuperclass().getDeclaredField("lastCompletedFilter");
			lastCompletedFilter.setAccessible(true);
			lastCompletedFilter.set(this, null);
		} catch (SecurityException e) {
			MessagingUtils.error(getClass(), e);
		} catch (NoSuchFieldException e) {
			MessagingUtils.error(getClass(), e);
		} catch (IllegalArgumentException e) {
			MessagingUtils.error(getClass(), e);
		} catch (IllegalAccessException e) {
			MessagingUtils.error(getClass(), e);
		}
		
		// this also is not the nicest way to do it. 
		// I am still amazed, that FilteredSelectionDialog does not offer any methods to change its data
		// once it was opened. Am I doing it wrong?
		String pattern = getTitle(cdmObject);
		((Text) getPatternControl()).setText(pattern);
	}
	
	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#fillContentProvider(org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.AbstractContentProvider, org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.ItemsFilter, org.eclipse.core.runtime.IProgressMonitor)
	*/
	/** {@inheritDoc} */
	@Override
	protected void fillContentProvider(AbstractContentProvider contentProvider,
		ItemsFilter itemsFilter, IProgressMonitor progressMonitor)
		throws CoreException {
		try {
			if(model != null){				
				progressMonitor.beginTask("Looking for entities", model.size());
				for(T element : model){
					contentProvider.add(element, itemsFilter);
					if (progressMonitor.isCanceled()) {
						throw new OperationCanceledException();
					}
					progressMonitor.worked(1);
				}
			}else{
				MessagingUtils.warn(getClass(), "Model for Filtered Selection is null:" + this.getClass().getSimpleName());
			}
		}
		finally {
			progressMonitor.done();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getDialogSettings()
	 */
	/** {@inheritDoc} */
	@Override
	protected IDialogSettings getDialogSettings() {
		IDialogSettings settings = TaxeditorStorePlugin.getDefault().getDialogSettings().getSection(getSettings());

		if (settings == null) {
			settings = TaxeditorStorePlugin.getDefault().getDialogSettings().addNewSection(getSettings());
		}
		return settings;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getElementName(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public String getElementName(Object item) {
		return ((IEnumTerm) item).getMessage();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#getItemsComparator()
	 */
	/** {@inheritDoc} */
	@Override
	protected Comparator getItemsComparator() {
		return new Comparator<IEnumTerm>() {
			public int compare(IEnumTerm entity1,
					IEnumTerm entity2) {
				Collator collator = Collator.getInstance();
				return collator.compare(entity1.getMessage(), entity2.getMessage());
			}
		};
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#validateItem(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	protected IStatus validateItem(Object item) {
		return Status.OK_STATUS;
	}
	
	/**
	 * <p>getSelectedUuidAndTitleCache</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.model.common.UuidAndTitleCache} object.
	 */
	protected T getSelectedEnumTerm() {
		Object[] result = getResult();
		return result[0] == null ? null : (T) result[0];
	}
	
	/**
	 * <p>Getter for the field <code>settings</code>.</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getSettings()  {
		if(settings == null){
			throw new IllegalStateException("No SETTINGS set.");
		}
		return settings;
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created Oct 19, 2009
	 * @version 1.0
	 */
	private class ResourceSelectionHistory extends SelectionHistory {
	    /*
	    * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#restoreItemFromMemento(org.eclipse.ui.IMemento)
	   	*/
		protected Object restoreItemFromMemento(IMemento element) {
			return element.getString("resource"); //$NON-NLS-1$
	  	}
	  	/*
	  	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog.SelectionHistory#storeItemToMemento(java.lang.Object,
	  	 *      org.eclipse.ui.IMemento)
	  	 */
		protected void storeItemToMemento(Object item, IMemento element) {
			element.putString("resource", item.toString()); //$NON-NLS-1$
		}
	}
	
	/**
	 * <p>getNewWizardLinkText</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	protected abstract String getNewWizardLinkText();
	
	/**
	 * <p>getNewEntityWizard</p>
	 * @param parameter 
	 * @return a {@link eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard} object.
	 */
	protected abstract AbstractNewEntityWizard getNewEntityWizard(String parameter);
	
	public class FilteredCdmResourceLabelProvider extends LabelProvider {
		public String getText(Object element) {
			if (element == null) {
				return null;
			}
			return ((IEnumTerm) element).getMessage();
		}			
	};

	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	/** {@inheritDoc} */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		if(getNewWizardLinkText() != null){
			Link link = new Link(parent, SWT.NONE);
			link.setText(getNewWizardLinkText());
			link.addSelectionListener(getNewWizardLinkSelectionListener());
			return link;
		}
		return null;
	}
	
	protected SelectionListener getNewWizardLinkSelectionListener(){
		return new SelectionAdapter() {
			
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				
				AbstractNewEntityWizard wizard = getNewEntityWizard(e.text);
				wizard.init(null, null);
				WizardDialog dialog = new WizardDialog(getShell(), wizard);
				int status = dialog.open();
				
				if (status == IStatus.OK) {
					
					T entity = (T) wizard.getEntity();

					refresh();
					setPattern(entity);
					getConversationHolder().bind();
				}
			}
		};
	}
	
	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	/** {@inheritDoc} */
	public void update(CdmDataChangeMap changeEvents) {}

}
