/**
 *
 */
package eu.etaxonomy.taxeditor.ui.element;

import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TypedListener;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.forms.widgets.ToggleHyperlink;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;

/**
 * <p>
 * Abstract super class for a {@link Section} GUI element that visualizes a CDM
 * entity, manages a conversation and listens to selections
 * </p>
 *
 * @param <ENTITY> A CDM entity which should be visualized by this section.
 *
 * @author n.hoffmann
 * @created Feb 22, 2010
 * @version 1.0
 * @param <T>
 */
//TODO shouldn't ENTITY be bound with super class ICdmBase for example (AbstractFormSection<ENTITY extends ICdmBase>)?
public abstract class AbstractFormSection<ENTITY> extends Section implements ISelectionChangedListener, IEntityElement<ENTITY>, IConversationEnabled {

    /**
     * The default number of columns in detail sections
     */
    public static final int DEFAULT_NUM_COLUMNS = 2;

	private ISelectionProvider selectionProvider;

	private ENTITY entity;

	private final Set<ICdmFormElement> elements = new HashSet<ICdmFormElement>();

	protected CdmFormFactory formFactory;

	private List<IPropertyChangeListener> propertyChangeListeners;

	private ICdmFormElement parentElement;

	private Color persistentBackgroundColor;

	/**
	 * <p>
	 * Constructor for AbstractFormSection.
	 * </p>
	 *
	 * @param conversation
	 *            TODO
	 * @param style
	 *            a int.
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param <ENTITY>
	 *            a ENTITY object.
	 */
    protected AbstractFormSection(CdmFormFactory formFactory, ICdmFormElement parentElement, int style) {
		super(parentElement.getLayoutComposite(), style);

		this.parentElement = parentElement;

		this.formFactory = formFactory;

		this.setLayoutData(LayoutConstants.FILL());

		Composite client = formFactory.createComposite(this, SWT.WRAP);
		client.setBackgroundMode(SWT.INHERIT_DEFAULT);

		TableWrapLayout layout = LayoutConstants.LAYOUT();
		layout.bottomMargin = 10;
		layout.rightMargin = 5;

		client.setLayout(layout);

		this.setClient(client);

	}

	    /**
     * <p>
     * Constructor for AbstractFormSection.
     * </p>
     *
     * @param formFactory
     *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
     *            object.
     * @param conversation
     *            a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
     *            object.
     * @param parentElement
     *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
     *            object.
     * @param selectionProvider
     *            a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
     * @param style
     *            a int.
     */
    protected AbstractFormSection(CdmFormFactory formFactory, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
        this(formFactory, parentElement, style);
        this.selectionProvider = selectionProvider;
    }

	/**
	 * <p>
	 * Getter for the field <code>propertyChangeListeners</code>.
	 * </p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	@Override
    public List<IPropertyChangeListener> getPropertyChangeListeners() {
		return propertyChangeListeners;
	}

	/** {@inheritDoc} */
	@Override
    public void setPropertyChangeListeners(
			List<IPropertyChangeListener> propertyChangeListeners) {
		this.propertyChangeListeners = propertyChangeListeners;
	}

	/**
	 * <p>
	 * Setter for the field <code>entity</code>.
	 * </p>
	 *
	 * @param entity
	 *            a ENTITY object.
	 */
	public void setEntity(ENTITY entity) {
		this.entity = entity;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.IEntityElement#getEntity()
	 */
	/**
	 * <p>
	 * Getter for the field <code>entity</code>.
	 * </p>
	 *
	 * @return a ENTITY object.
	 */
	@Override
    public ENTITY getEntity() {
		return entity;
	}

	/**
	 * <p>
	 * getToggle
	 * </p>
	 *
	 * @return a {@link org.eclipse.ui.forms.widgets.ToggleHyperlink} object.
	 */
	public ToggleHyperlink getToggle() {
		return this.toggle;
	}

	/**
	 * <p>
	 * getSection
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *         object.
	 */
	public AbstractFormSection<ENTITY> getSection() {
		return this;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent
	 * ()
	 */
	/** {@inheritDoc} */
	@Override
    public void firePropertyChangeEvent(CdmPropertyChangeEvent event) {
        Assert.isNotNull(propertyChangeListeners, "No property change listeners.");
        try {
            for (Object listener : propertyChangeListeners) {
                ((IPropertyChangeListener) listener).propertyChange(event);
            }
        } catch (ConcurrentModificationException e) {
            MessagingUtils.warn(getClass(), "ConcurrentModificationException while handling PropertyChangeEvents."
                    + " It seems like this is not critical");
        }
	}

	/**
	 * Fires a {@link CdmPropertyChangeEvent} with the given object as source.
	 *
	 * @param object
	 *            the object on which the property changed
	 */
	public void firePropertyChangeEvent(Object object) {
		firePropertyChangeEvent(object, null);
	}

	/**
	 * Fires a {@link CdmPropertyChangeEvent} with the given object as source
	 * also containing the originating event
	 *
	 * @param object
	 *            the object on which the property changed
	 * @param originatingEvent
	 *            the originating event
	 */
	public void firePropertyChangeEvent(Object object,
			PropertyChangeEvent originatingEvent) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(object,
				originatingEvent));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.eclipse.swt.widgets.Composite#setFocus()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean setFocus() {
		return getClient().setFocus();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse
	 * .jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
    public void propertyChange(PropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics
	 * .Color)
	 */
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		for (ICdmFormElement element : getElements()) {
			element.setBackground(color);
		}
		getLayoutComposite().setBackground(color);
		super.setBackground(color);
	}

	@Override
	public void setPersistentBackground(Color color) {
		persistentBackgroundColor = color;
		setBackground(color);
	}

	@Override
	public Color getPersistentBackground() {
		return persistentBackgroundColor;
	}


	/**
	 * <p>
	 * widgetSelected
	 * </p>
	 *
	 * @param e
	 *            a {@link org.eclipse.swt.events.SelectionEvent} object.
	 */
	public void widgetSelected(SelectionEvent e) {
        Widget widget = e.widget;

        if (widget instanceof Control) {
            Control control = (Control) widget;
            if (checkControlAncestryForWidget(control)) {
                if (getEntity() != null) {
                    IStructuredSelection selection = new StructuredSelection(getEntity());
                    if (selectionProvider != null) {
                        selectionProvider.setSelection(selection);
                    }
                }
            }
        }
	}

	private boolean checkControlAncestryForWidget(Control control) {
		if (control.equals(this)) {
			return true;
		} else {
			Control parent = control.getParent();
			if (parent == null) {
				return false;
			} else {
				return checkControlAncestryForWidget(parent);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
    public void setSelected(boolean selected) {
		if (selected) {
			setBackground(Display.getCurrent().getSystemColor(
					SWT.COLOR_LIST_SELECTION));
		} else {
			setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		}
	}

	/** {@inheritDoc} */
	@Override
    public void selectionChanged(SelectionChangedEvent event) {
		if (event.getSelection() == CdmFormFactory.EMPTY_SELECTION) {
			return;
		}

		IStructuredSelection selection = (IStructuredSelection) event
				.getSelection();
		setSelected(false);

		Object selectedObject = selection.getFirstElement();

		if (selectedObject != null && selectedObject.equals(getEntity())) {
			setSelected(true);
		}
	}

	/**
	 * <p>
	 * addSelectionListener
	 * </p>
	 *
	 * @param listener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void addSelectionListener(SelectionListener listener) {
		addListener(SWT.Selection, new TypedListener(listener));
	}

	/**
	 * <p>
	 * removeSelectionListener
	 * </p>
	 *
	 * @param listener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void removeSelectionListener(SelectionListener listener) {
		removeListener(SWT.Selection, listener);
	}

	/** {@inheritDoc} */
	@Override
    public void addElement(ICdmFormElement element) {
		elements.add(element);
	}

	/**
	 * <p>
	 * removeElement
	 * </p>
	 *
	 * @param element
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 */
	protected void removeElement(ICdmFormElement element) {
		elements.remove(element);
	}

	/**
	 * <p>
	 * removeElements
	 * </p>
	 */
	@Override
    public void removeElements() {
		for (ICdmFormElement childElement : getElements()) {
			// recursion
			childElement.removeElements();

			// unregister selection arbitrator
			if (childElement instanceof ISelectableElement) {
				ISelectableElement selectableElement = (ISelectableElement) childElement;
				if (selectableElement.getSelectionArbitrator() != null) {
					formFactory.destroySelectionArbitrator(selectableElement
							.getSelectionArbitrator());
				}
			}

			// unregister propertyChangeListener
			formFactory.removePropertyChangeListener(childElement);

			// dispose of the controls
			for (Control control : childElement.getControls()) {
				// we added the layoutComposite of the parental element as the
				// layout composite to this formElement
				// but we do not want to destroy it.
				if (control.equals(childElement.getLayoutComposite())) {
					continue;
				} else {
					control.dispose();
					control = null;
				}
			}
		}

		elements.clear();
	}

	/**
	 * <p>
	 * Getter for the field <code>parentElement</code>.
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	@Override
    public ICdmFormElement getParentElement() {
		return parentElement;
	}

	/**
	 * <p>
	 * Getter for the field <code>elements</code>.
	 * </p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	@Override
    public Set<ICdmFormElement> getElements() {
		return elements;
	}

	/**
	 * <p>
	 * getControls
	 * </p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	@Override
    public Set<Control> getControls() {
		Set<Control> controls = new HashSet<Control>();

		for (Control control : getChildren()) {
			controls.add(control);
		}

		return controls;
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		removeElements();
		super.dispose();
	}

	/**
	 * <p>
	 * getLayoutComposite
	 * </p>
	 *
	 * @return a {@link org.eclipse.swt.widgets.Composite} object.
	 */
	@Override
    public Composite getLayoutComposite() {
		return (Composite) getClient();
	}

	/** {@inheritDoc} */
	@Override
    public boolean containsFormElement(ICdmFormElement formElement) {
		if (formElement == this) {
			return true;
		} else {
			for (ICdmFormElement element : getElements()) {
				boolean contains = element.containsFormElement(formElement);
				if (contains == true) {
					return true;
				}
			}
			return false;
		}
	}

	/**
	 * <p>
	 * Getter for the field <code>formFactory</code>.
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 */
	@Override
    public CdmFormFactory getFormFactory() {
		return formFactory;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormElement#refresh()
	 */
	/**
	 * <p>
	 * refresh
	 * </p>
	 */
	@Override
    public void refresh() {
		// empty default implementation

	}

	/**
	 * <p>
	 * getConversationHolder
	 * </p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
	 *         object.
	 */
	@Override
    public ConversationHolder getConversationHolder() {
		if(getParentElement() instanceof RootElement || getParentElement() == null){

			IEditorPart activeEditor = AbstractUtility.getActiveEditor();
			if(activeEditor instanceof IConversationEnabled){
				ConversationHolder conversation = ((IConversationEnabled) AbstractUtility.getActiveEditor()).getConversationHolder();
				return conversation;
			}
		}else if(getParentElement() instanceof IConversationEnabled){
			return ((IConversationEnabled) getParentElement()).getConversationHolder();
		}
		MessagingUtils.messageDialog("Could not get conversation for AbstractFormSection",
				getClass(), "There is an error in the implementation. There should have been an active editor but it wasn't",
				new IllegalArgumentException());
		return null;

	}

	/** {@inheritDoc} */

	@Override
    public void update(CdmDataChangeMap changeEvents) {

	}


}
