// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section;

import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;

/**
 * This class visualizes an CDM entity of type ENTITY.
 *
 * @param <ENTITY> A CDM entity which should be visualized by this section.
 *
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDetailSection<ENTITY> extends AbstractFormSection<ENTITY> implements IEnableableFormElement, IExpansionListener {

	protected ICdmDetailElement<ENTITY> detailElement;

	public AbstractCdmDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
	    this(formFactory, null, conversation, parentElement, selectionProvider, style);
	}


	public AbstractCdmDetailSection(CdmFormFactory formFactory, Class<ENTITY> clazz,
			ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
		super(formFactory, parentElement, selectionProvider,
				ExpandableComposite.CLIENT_INDENT | style);

        setText(getHeading());

        addExpansionListener(this);

        if(clazz==null){
            createControls(this, style);
        }
        else{
            createControlsByType(this, clazz, SWT.NULL);
        }
	}

	protected void createControlsByType(AbstractCdmDetailSection<ENTITY> formElement, Class<ENTITY> entityClass, int style) {
	    TableWrapLayout layout = (TableWrapLayout) getLayoutComposite().getLayout();
	    layout.topMargin = 10;
	    layout.numColumns = DEFAULT_NUM_COLUMNS;

	    getLayoutComposite().setLayout(layout);
	    if(entityClass==null){
	        detailElement = createCdmDetailElement(formElement, style);
	    }
	    else{
	        detailElement = createCdmDetailElementByType(formElement, entityClass, style);
	    }
	}

	protected void createControls(AbstractCdmDetailSection<ENTITY> formElement, int style) {
	    createControlsByType(formElement, null, style);
	}

	protected abstract ICdmDetailElement<ENTITY> createCdmDetailElement(AbstractCdmDetailSection<ENTITY> parentElement, int style);

	protected ICdmDetailElement<ENTITY> createCdmDetailElementByType(AbstractCdmDetailSection<ENTITY> parentElement, Class<ENTITY> entityClass, int style){
	    return createCdmDetailElement(parentElement, style);
	}

	public abstract String getHeading();

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		if (detailElement instanceof ISelectableElement) {
			ISelectableElement selectableElement = (ISelectableElement) detailElement;
			if (selectableElement.getSelectionArbitrator() != null) {
				formFactory.destroySelectionArbitrator(selectableElement
						.getSelectionArbitrator());
			}
		}
		super.dispose();
	}

	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		if (detailElement != null) {
			// detailComposite.setBackground(color);
		}
		super.setBackground(color);
	}

	@Override
	public void setEntity(ENTITY entity) {
		if (detailElement != null) {
			detailElement.setEntity(entity);
		}
		super.setEntity(entity);
		setSectionTitle();
		layout();
	}

	protected void setSectionTitle() {
		String title = "";
		if (getEntity() != null && (getEntity() instanceof IdentifiableEntity) && !(getEntity() instanceof SpecimenOrObservationBase)) {
			title = ": " + ((IdentifiableEntity) getEntity()).getTitleCache();
			// we have to duplicate ampersands otherwise they are treated as
			// mnenomic (see Label.setText() documentation)
			// see also #4302
			title = title.replace("&", "&&");
		}
		this.setText(String.format("%s%s", getHeading(), title));
		setTextClient(createToolbar());
	}

	protected Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
		return toolBarManager.createControl(this);
	}

	public void updateTitle() {
		if (!isDisposed()) {
			setSectionTitle();
			layout();
		}
	}

	/** {@inheritDoc} */
	@Override
    public void setIrrelevant(boolean irrelevant) {
		if (detailElement != null) {
			detailElement.setIrrelevant(irrelevant);
		}
	}

	/** {@inheritDoc} */
	@Override
    public void expansionStateChanging(ExpansionEvent e) {
		// logger.warn("Expansion State Changing");
	}

	/** {@inheritDoc} */
	@Override
    public void expansionStateChanged(ExpansionEvent e) {
		// logger.warn("Expansion State Changed");
	}
}
